Attribute VB_Name = "EqParser_Fnct"
Option Explicit
Option Base 1
'EqParser Supported Functions and Operators
'==========================================
'Functions: abs, exp, fix, int, log, ln, log10, rnd=(Random(arg)), sgn,  sqr
'           es. ln(4) = log(4) returns 1.3862...
'Trigon Functions
'           atan, sin, tan, cos, cosec, cotan, sec, arcsin, arccos, arcsec, arccos, arccotan
'           es. sin(1.2) returns 0.9320...
'Hyperbolic functions
'           sinh, cosh, tanh, sech, cosech, cotanh, arcsinh, arccosh, arctanh, arcsech, arccosech, arccotanh
'           es. sinh(1.2) returns 1.50946...
'Operators: and, eqv, imp, mod, not, or, xor
'           es. (1 and 2) returns 1
'Stat Functions: min, max, avg, sum, prod, sum2 (=sum^2), prod2 (=prod^2), stddev, count, geom_mean, median, mean
'                es. min(1;2;3;0.5) returns 0.5
'
'Matrix Operators: det trace syst vcoeff (es. det([mA]) calculates the determinant of [mA])
'
'EqParser Returned values (ExitCodes) and Constants
'==================================================
Global Const e_evalOK = -1              'no error
Global Const e_NO_ERROR = -1
Global Const e_AmbFunctName = -2        'Ambiguous function name detected
Global Const e_SyntaxError_eval = -3    'syntax error
Global Const e_SyntaxError_parser = -4  'syntax error
Global Const e_UndefFunct = -5          'Undefined function
Global Const e_RetValNotDef = -6        'Returned Value not defined
Global Const e_TokenMatchError = -7
Global Const e_NotFound = -8
Global Const e_DivByZero = -9
Global Const e_ExpNotInt = -10
Global Const e_CondNotFound = -13
Global Const e_WrongStringMatrix = -14
Global Const e_NoEnoughtElements = -15
Global Const e_InvalidParam = -16
Global Const e_MatrNotFound = -17
Global Const e_NotRegistered = -18
Global Const NotEvenNum = -11  'Simpson's Rule requires that number of subintervals N be even
Global Const MemAllocErr = -12
Global Const MatrNotFound = -13
Global Const e_IsNumeric = 0   'no error and result is numeric
Global Const nnIsNumber = 0
Global Const nnIsMatrix = 1
'Constants for FindMatrix
'------------------------
Global Const RetMatrPtr = 1
Global Const RetIndex = 2
Global Const SrcMatrPtr = 8
Global Const SrcName = 32

'; get value of internal e_error variable (se constants for values)
Public Declare Function GetE_Error Lib "aNuMet.dll" () As Long

'
Public Declare Function ClearMatrixList Lib "aNuMet.dll" ( _
       aPtr  As Any) As Long
' clear list created by SetMatrix in memory
' *aPtr is a pointer to a Matrix to skip deletion
' if *aPtr=0 then delete all, otherwise only external matrices are remmoved

';
Public Declare Function ClearVarFuncList Lib "aNuMet.dll" ( _
       ByVal Tp As String) As Long
'  Tp.s="vars" clear Variables' List, otherwise clear Functions' list

';calculate n!, factorial of n (es. n! = 5! = 1*2*3*4*5 = 120); return a double
Public Declare Function fact Lib "aNuMet.dll" ( _
       ByVal n As Long) As Double

'
Public Declare Function FindMatrix Lib "aNuMet.dll" ( _
       ByVal MatrName As String,  _
       ByVal Tp As Long,  _
       rows  As Long,  _
       cols  As Long) As Long
' search for Matrix in the internal list/memory.
' If found, Return values varies With parameter Tp, if Matrix is not found return #MatrNotFound (-17)
' #SrcMatrPtr = 8  search by Pointer to Matrix
' #SrcName    = 32 use name (es. [mA]) as string to find
' #RetMatrPtr = 1  Return Pointer To Matrix
' #RetIndex   = 2  Return the index/position of the element in the list
' dimensions are always returned (if Matrix found);
' this function Trim spaces of MatrName

'; return content of a matrix stored internally
Public Declare Function GetMatrixByLong Lib "aNuMet.dll" ( _
       ByVal SrcMemID As Long,  _
       aDest As Any,  _
       ByVal rows As Long,  _
       ByVal cols As Long) As Long
' used in lieu of aCopyArray to have "As Long" in the VB Function declaration

';; get list created by SetMatrix in memory; return a pointer to a string
Public Declare Function GetMatrixList Lib "aNuMet.dll" ( _
       ByVal ShowElem As Byte) As Long
' *aPtr is a pointer to a Matrix to skip deletion
' if *aPtr=0 then delete all, otherwise only external matrices are remmoved

';return 0 (#nnIsNumber) if result of e_eval(...*dRetVal...) is a Number; 1 (#nnIsMatrix) if it is a Matrix
Public Declare Function GetResultType Lib "aNuMet.dll" () As Long
'see sample code to know how recover Result Matrix

';get VarFuncList string; result pointed by *sList
Public Declare Function GetVarFuncList Lib "aNuMet.dll" ( _
       ByVal sList As String,  _
       ByVal sListLen As Long,  _
       ByVal Tp As String) As Long
' Tp.s="vars" get Variables' List, otherwise get Functions' list

';get Lenght of VarsList
Public Declare Function GetVarFuncListLen Lib "aNuMet.dll" ( _
       ByVal Tp As String) As Long
' Tp.s="vars" get Variables' List, otherwise get Functions' list

';reset all internal variables used by e_eval function
Public Declare Function ReSetEqParser Lib "aNuMet.dll" () As Long

';set internal sign returned by boolean operations (i.e. 1 or -1)
Public Declare Function SetBoolSign Lib "aNuMet.dll" ( _
       ByVal kk As Long) As Long

';set/store a matrix in the internal matrix list
Public Declare Function SetMatrix Lib "aNuMet.dll" ( _
       ByVal MatrName As String,  _
       aPtr As Any,  _
       ByVal rows As Long,  _
       ByVal cols As Long,  _
       ByVal Return_MatrPtr As Byte) As Long
' matrix can then be found with FindMatrix function by name (es. "[mA]") or by string of the Pointer (es. Str(@mA())
' if Return_MemID = 0, return LinkedList Position, if Return_MemID <> 0 return pointer to matrix

';set/store a string matrix in the internal matrix list
Public Declare Function SetStrMatrix Lib "aNuMet.dll" ( _
       ByVal StrMatr As String,  _
       ByVal Return_mPtr As Byte,  _
       ByVal e_evalF As Byte) As Long
' returned value is: Matrix List position if Return_mPtr = 0, otherwise is pointer to matrix values
' if e_evalF <> 0 then elements contains expressions and e_eval function is used to evaluate them statically (once a run)
'                      name  r; c; values
'es. of string matrix "[mA]; 3; 3; 0; 1; 2; 1.2; 2.2; 3.2; 1; 2; 12*sin(0.5)" with function
'es. of string vector "[vB]; 3; 0; 0; 1; 2"; this is a vector
' matrix can then be found with FindMatrix function by name (es. "[mA]") or by string of the Pointer (es. Str(@mA())

'; Add or Update the value of a Variable or a Function in the relevant Internal List
Public Declare Function SetVarFunct Lib "aNuMet.dll" ( _
       ByVal sVar As String,  _
       ByVal sVal As String,  _
       ByVal v0f1 As Long) As Long
' return number of items, es. "xxx=1.234;yyy=2.22;xyz=333.22"

'; set to show error and debug messages during e_eval routine program execution
Public Declare Function ShowMsgs Lib "aNuMet.dll" ( _
       ByVal bShow As Long) As Long
'Parameters:   0 = No Messages
'              1 = Show Error Messages
'              2 = Show Debug Messages
'              3 = Show All Messages

';evaluate a string containing an expression.
Public Declare Function e_eval Lib "aNuMet.dll" ( _
       ByVal expression As String,  _
       dRetVal As Double,  _
       ByVal ChkFunct As Long) As Long
'If successful, return true and place result in dRetVal.

'
Public Declare Function e_evals Lib "aNuMet.dll" ( _
       ByVal lExpr As String,  _
       ByVal lx0 As Double,  _
       ByVal lsVar As String,  _
       ByVal lxN As Double,  _
       ByVal lN1 As Long,  _
       Xv As Long,  _
       Yv As Long,  _
       MinMaxXY As Long) As Long
'execute e_eval lN1 times
'return three vectors: Xv.d(lN1), Yv.d(lN1) and MinMaxXY.d(4)
'  MinMaxXY(1) = MinXv: MinMaxXY(2) = MinYv: MinMaxXY(3) = MaxXv: MinMaxXY(4) = MaxYv
'return -1 if N errors, otherwise error code

